#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "lcd.h"
#include "usart.h"
#include "adc.h"

#define F_CPU 16000000
#define HALL_TH 100
#define PI 3.145
// 26 * 2.54 /2 = 33.02 cm
#define DEFAULT_WHEEL 33.02

#define first_line 0x0
#define second_line 0x40

#define BUFSIZE 16 

#define MAGN 50
#define NO_MAGN 1000

#define S 0
#define T 1
#define R 2
#define D 3

static char old_d_int = 0;
static int rpm = 0;
static int speed_ms = 0;
static int speed_kmh = 0;
static int distance = 0;
static int time = 0;
static char buffer[BUFSIZE];

static char select_function = 0;

// initializez timer care genereaza intrerupere la 1s
void timer1_init()
{
	TCCR1B |= (1 << CS12)| (1 << CS10) | (1 << WGM12);

	TIMSK1 |= (1 << OCIE1A);

	OCR1A = 15625;
}

// MISC
int get_speed_ms (int rpm)
{
	return (rpm / 60) * (2 * PI * DEFAULT_WHEEL / 100);
}

int get_speed_kmh (int rpm)
{
	return get_speed_ms(rpm) * 3600 / 1000;
}

int get_hours (int time)
{
	return (int) time / 3600;
}

int get_minutes (int time)
{
	return (int) (time % 3600) / 60;
}

int get_seconds (int time)
{
	return (int) (time % 3600) % 60;
}

// Interrupts from buttons
ISR(PCINT3_vect) {
	if ((PIND & (1 << PD0)) == 0 && (old_d_int & (1 << PD0)) != 0) {
		select_function = 0;
		select_function |= (1 << S);
	} else if ((PIND & (1 << PD6)) == 0 && (old_d_int & (1 << PD6)) != 0) {
		select_function = 0;
		select_function |= (1 << T);
	} else if ((PIND & (1 << PD4)) == 0 && (old_d_int & (1 << PD4)) != 0) {
		select_function = 0;
		select_function |= (1 << R);
	} else if ((PIND & (1 << PD5)) == 0 && (old_d_int & (1 << PD5)) != 0) {
		select_function = 0;
		select_function |= (1 << D);
	}

	old_d_int = PIND;
	_delay_ms(10);
}

// Interrupt handler for timer
ISR (TIMER1_COMPA_vect) {
	// Blinky
	PORTD ^= (1 << PD7);

	time += 1;
	distance = get_speed_ms(rpm) * time;

	if((select_function & (1 << S)) != 0){
		// USART0_print("INT Button 0 pressed\n");
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		LCD_DELAY();
		sprintf(buffer, "%d", speed_ms);
		LCD_print("Speed m/s");
		LCD_printAt(second_line, buffer);
		LCD_print(" m/s");
	}
	if((select_function & (1 << T)) != 0){
		// USART0_print("INT Button 1 pressed\n");
		int hours = get_hours(time);
		int minutes = get_minutes(time);
		int seconds = get_seconds(time);
		sprintf(buffer, "%d:%d:%d", hours, minutes, seconds);
		// USART0_print(buffer);
		// USART0_print("\n");
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		LCD_DELAY();
		LCD_print("Time");
		LCD_printAt(second_line, buffer);
	}
	if((select_function & (1 << R)) != 0){
		// USART0_print("INT Button 2 pressed\n");
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		LCD_DELAY();
		sprintf(buffer, "%d", rpm);
		LCD_print("RPM");
		LCD_printAt(second_line, buffer);
		LCD_print(" rot/minut");
	}
	if((select_function & (1 << D)) != 0){
		// USART0_print("INT Button 3 pressed\n");
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		LCD_DELAY();
		sprintf(buffer, "%d", distance);
		LCD_print("Distance");
		LCD_printAt(second_line, buffer);
		LCD_print(" m");
	}

}

int main() {
	char str[] = "Start program";
	uint16_t adc_result;
	uint16_t last_adc_result = 1024;

	int counter = 0;

	sei();

	// Timer 1
	TCCR1B |= (1 << CS12);		// prescaler clk : 256
	TIMSK1 |= (1 << TOIE1);

	TCCR1B |= (1 << WGM12);		// CTC mode, OCR1A
	TIMSK1 |= (1 << OCIE1A);	// Enable interrupts 

	// 1 second
	OCR1A = F_CPU / 256;

	// Buttons
	PCMSK3 |= (1 << PCINT24);
	PCMSK3 |= (1 << PCINT30);
	PCMSK3 |= (1 << PCINT28);
	PCMSK3 |= (1 << PCINT29);
	PCICR |= (1 << PCIE3);

	// Init pins
	DDRD |= (1 << PD7); // LED
	DDRD &= ~(1 << PD0); // Buton 0
	DDRD &= ~(1 << PD6); // Buton 1
	DDRD &= ~(1 << PD4); // Buton 2
	DDRD &= ~(1 << PD5); // Buton 3

	// Pull-up resistors activated
	PORTD |= (1 << PD0);
	PORTD |= (1 << PD6);
	PORTD |= (1 << PD4);
	PORTD |= (1 << PD5); 

	// ADC
	adc_init();

	// LCD
	LCD_init();
	LCD_print(str);

	// USART
	USART0_init();
	USART0_print(str);
	_delay_ms(50);
 
	while(1)
	{
      	// read adc value at PA0
		adc_result = adc_read(PA0);

		// Get data from Hall sensor
		if ((int) adc_result < HALL_TH) {
			if(last_adc_result - adc_result > HALL_TH) {
				counter += 1;	
			}
		}
		
		char test_buffer[BUFSIZE];
		sprintf(test_buffer, "%d %d %d\n", adc_result, last_adc_result, counter);
 		USART0_print(test_buffer);

 		last_adc_result = adc_result;

 		// TODO Compute rpm
		rpm = 90;

 		// Compute speed
 		speed_ms = get_speed_ms(rpm);
		speed_kmh = get_speed_kmh(rpm);
	}

	return 0; // don't let the compiler complain about this
}


